package com.huimv.manager.dao.repo;

import com.huimv.manager.dao.entity.ImagePathEntity;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Query;

import java.util.List;

public interface ImagePathRepo extends JpaRepository<ImagePathEntity, Integer>, JpaSpecificationExecutor<ImagePathEntity> {

    /**
     * @Method      : getLastImage
     * @Description : 读取最新的图像记录
     * @Params      : []
     * @Return      : com.huimv.manager.dao.entity.ImagePathEntity
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/1/22       
     * @Time        : 10:13
     */
    @Query(nativeQuery = true, value = "SELECT * FROM image_path ORDER BY create_date DESC LIMIT 1")
    ImagePathEntity getLastImage();

    /**
     * @Method      : getNextImage
     * @Description : 读取下一幅有效的图像记录
     * @Params      : []
     * @Return      : com.huimv.manager.dao.entity.ImagePathEntity
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/1/22       
     * @Time        : 10:13
     */
    @Query(nativeQuery = true, value = "SELECT * FROM image_path WHERE effective=1 ORDER BY create_date ASC LIMIT 1")
    ImagePathEntity getNextImage();

    @Query(nativeQuery = true, value = "SELECT id,name,img_path,relative_path,file_name,effective,ai_result,ai_time_used,ai_data,create_date,pig_quantity FROM image_path WHERE DATE_FORMAT(create_date,'%Y-%m-%d')>=DATE_FORMAT(:startDate,'%Y-%m-%d') AND DATE_FORMAT(create_date,'%Y-%m-%d')<=DATE_FORMAT(:endDate,'%Y-%m-%d') ORDER BY create_date DESC LIMIT :pageNo,:pageSize")
    List<ImagePathEntity> listImage(String startDate, String endDate,int pageNo,int pageSize);

    @Query(nativeQuery = true, value = "SELECT id,name,img_path,relative_path,file_name,effective,ai_result,ai_time_used,ai_data,create_date,pig_quantity FROM image_path WHERE DATE_FORMAT(create_date,'%Y-%m-%d')>=DATE_FORMAT(:startDate,'%Y-%m-%d') AND DATE_FORMAT(create_date,'%Y-%m-%d')<=DATE_FORMAT(:endDate,'%Y-%m-%d') AND name=:catalog ORDER BY create_date DESC LIMIT :pageNo,:pageSize")
    List<ImagePathEntity> listImage(String startDate, String endDate,int pageNo,int pageSize,String catalog);


    @Query(nativeQuery = true, value = "SELECT id,name,img_path,relative_path,file_name,effective,ai_result,ai_time_used,ai_data,create_date,pig_quantity FROM image_path WHERE DATE_FORMAT(create_date,'%Y-%m-%d')>=DATE_FORMAT(:startDate,'%Y-%m-%d') AND DATE_FORMAT(create_date,'%Y-%m-%d')<=DATE_FORMAT(:endDate,'%Y-%m-%d') ORDER BY create_date DESC LIMIT :pageNo,:pageSize")
    List<ImagePathEntity> listImage2(String startDate, String endDate,int pageNo,int pageSize);

    @Query(nativeQuery = true, value = "SELECT count(id) FROM image_path WHERE DATE_FORMAT(create_date,'%Y-%m-%d')>=DATE_FORMAT(:startDate,'%Y-%m-%d') AND DATE_FORMAT(create_date,'%Y-%m-%d')<=DATE_FORMAT(:endDate,'%Y-%m-%d')")
    int getTotal(String startDate, String endDate);

    @Query(nativeQuery = true, value = "SELECT count(id) FROM image_path WHERE DATE_FORMAT(create_date,'%Y-%m-%d')>=DATE_FORMAT(:startDate,'%Y-%m-%d') AND DATE_FORMAT(create_date,'%Y-%m-%d')<=DATE_FORMAT(:endDate,'%Y-%m-%d') AND name=:catalog")
    int getTotal(String startDate, String endDate,String catalog);

    /**
     * @Method      : getExportImageList
     * @Description : 读取导出的图片
     * @Params      : [startDate, endDate]
     * @Return      : java.util.List
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/1/23       
     * @Time        : 17:06
     */
    @Query(nativeQuery = true, value = "SELECT * FROM image_path WHERE DATE_FORMAT(create_date,'%Y-%m-%d')>=DATE_FORMAT(:startDate,'%Y-%m-%d') AND DATE_FORMAT(create_date,'%Y-%m-%d')<=DATE_FORMAT(:endDate,'%Y-%m-%d')")
    List<ImagePathEntity> getExportImageListByStartAndEndDate(String startDate, String endDate);

    /**
     * @Method      : getExportImageListByStartAndEndDate
     * @Description : 读取指定目录的图片
     * @Params      : [startDate, endDate, catalog]
     * @Return      : java.util.List<com.huimv.manager.dao.entity.ImagePathEntity>
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/1/26       
     * @Time        : 19:03
     */
    @Query(nativeQuery = true, value = "SELECT * FROM image_path WHERE DATE_FORMAT(create_date,'%Y-%m-%d')>=DATE_FORMAT(:startDate,'%Y-%m-%d') AND DATE_FORMAT(create_date,'%Y-%m-%d')<=DATE_FORMAT(:endDate,'%Y-%m-%d') AND name=:catalog")
    List<ImagePathEntity> getExportImageListByStartAndEndDate(String startDate, String endDate,String catalog);

    /**
     * @Method      : getExportImageListById
     * @Description : 
     * @Params      : [imageIdList]
     * @Return      : java.util.List
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/1/23       
     * @Time        : 17:07
     */
    @Query(nativeQuery = true, value = "SELECT * FROM image_path WHERE id IN(:imageIdList)")
    List<ImagePathEntity> getExportImageListById(List imageIdList);

    /**
     * @Method      : getNextImageById
     * @Description : 读取下一张图片
     * @Params      : [imageId]
     * @Return      : void
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/1/23       
     * @Time        : 19:23
     */
    @Query(nativeQuery = true, value = "SELECT * FROM image_path WHERE id>:imageId ORDER BY id ASC LIMIT 1")
    ImagePathEntity getNextImageById(int imageId);

    /**
     * @Method      : getPreviousImageById
     * @Description : 读取前一张图片
     * @Params      : [imageId]
     * @Return      : void
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/1/23       
     * @Time        : 19:24
     */
    @Query(nativeQuery = true, value = "SELECT * FROM image_path WHERE id<:imageId ORDER BY id DESC LIMIT 1")
    ImagePathEntity getPreviousImageById(int imageId);

    /**
     * @Method      : getOneImageById
     * @Description : 
     * @Params      : [imageId]
     * @Return      : com.huimv.manager.dao.entity.ImagePathEntity
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/1/23       
     * @Time        : 19:45
     */
    @Query(nativeQuery = true, value = "SELECT * FROM image_path WHERE id=:imageId")
    ImagePathEntity getOneImageById(String imageId);

    /**
     * @Method      : getNextImageById
     * @Description : 带有目录的下一张图片
     * @Params      : [imageId, catalog]
     * @Return      : com.huimv.manager.dao.entity.ImagePathEntity
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/1/28       
     * @Time        : 9:18
     */
    @Query(nativeQuery = true, value = "SELECT * FROM image_path WHERE name=:catalog AND id>:imageId ORDER BY id ASC LIMIT 1")
    ImagePathEntity getNextImageById(int imageId, String catalog);

    /**
     * @Method      : getPreviousImageById
     * @Description : 带有目录的前一张图片
     * @Params      : [imageId, catalog]
     * @Return      : com.huimv.manager.dao.entity.ImagePathEntity
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/1/28       
     * @Time        : 9:19
     */
    @Query(nativeQuery = true, value = "SELECT * FROM image_path WHERE name=:catalog AND id<:imageId ORDER BY id DESC LIMIT 1")
    ImagePathEntity getPreviousImageById(int imageId, String catalog);

    @Query(nativeQuery = true, value = "SELECT * FROM image_path WHERE  id>=:id")
    List<ImagePathEntity> getAllUpdateImage(Integer id);

    @Query(nativeQuery = true, value = "SELECT * FROM image_path WHERE  id=:id")
    List<ImagePathEntity> getOneUpdateImage(Integer id);
}